package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Message;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.MessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/message")
@Api(description = "站内消息")
public class MessageController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MessageService messageService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Message> create(){
        MessageResult<Message> msgResult = new MessageResult<>();

        Message message = new Message();

        msgResult.setData(message);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<Message> add(@RequestBody Message message,@RequestAttribute String subject){
        MessageResult<Message> msgResult = new MessageResult<>();

        try {
            message.setId(UUID.randomUUID().toString());
            message.setDelFlag(false);
            message.setCreateBy(subject);
            message.setCreateTime(new Date());
            
            int affectCount = messageService.insert(message);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(message);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Message> edit(@PathVariable("id") String id){
        MessageResult<Message> msgResult = new MessageResult<>();

        try {
            Message message = messageService.get(id);

            if (message != null) {
                msgResult.setResult(true);
                msgResult.setData(message);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<Message> update(@RequestBody Message message,@RequestAttribute String subject){
        MessageResult<Message> msgResult = new MessageResult<>();

        try {
            message.setUpdateBy(subject);
            message.setUpdateTime(new Date());
            
            int affectCount = messageService.update(message);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(message);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Message message = messageService.get(id);
            message.setDelFlag(true);
            message.setUpdateBy(subject);
            message.setUpdateTime(new Date());

            int affectCount = messageService.update(message);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Message message = messageService.get(id);
                message.setDelFlag(true);
                message.setUpdateBy(subject);
                message.setUpdateTime(new Date());

                affectCount += messageService.update(message);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<Message> page = messageService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="消息列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<Message>> list(@RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<Message>> msgResult = new MessageResult<>();

        List<Message> list = messageService.findByRecipientId(subject);


        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
