package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.business.entity.ApplicationPolicy;
import com.jpsoft.picc.modules.business.service.ApplicationPolicyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/applicationPolicy")
@Api(description = "投保单和每月投保单关联表")
public class ApplicationPolicyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ApplicationPolicy> create(){
        MessageResult<ApplicationPolicy> msgResult = new MessageResult<>();

        ApplicationPolicy applicationPolicy = new ApplicationPolicy();

        msgResult.setData(applicationPolicy);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<ApplicationPolicy> page = applicationPolicyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="第一次投保除外的每月投保单列表")
    @RequestMapping(value = "otherPageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> otherPageList(
            String applicationId,String status,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(applicationId)) {
            searchParams.put("applicationId",applicationId);
        }

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        Page<ApplicationPolicy> page = applicationPolicyService.otherSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="每月投保单详情")
    @RequestMapping(value = "detail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<ApplicationPolicy> detail(String applicationId,String policyId,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<ApplicationPolicy> msgResult = new MessageResult<>();

        ApplicationPolicy applicationPolicy = applicationPolicyService.getDetail(applicationId,policyId);

        msgResult.setResult(true);
        msgResult.setData(applicationPolicy);

        return msgResult;
    }


    @ApiOperation(value="获取每月投保单列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query")
    })
    @RequestMapping(value = "findByApplicationId",method = RequestMethod.POST)
    public MessageResult<List<ApplicationPolicy>> findByApplicationId(
            String applicationId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<ApplicationPolicy>> msgResult = new MessageResult<>();

        List<ApplicationPolicy> list = applicationPolicyService.findByApplicationId(applicationId);

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

}
