package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.base.entity.Jobs;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import com.jpsoft.picc.modules.base.service.InsuranceJobsService;
import com.jpsoft.picc.modules.base.service.JobsService;
import com.jpsoft.picc.modules.business.entity.ApplicationPolicy;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.service.ApplicationPolicyService;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/insuranceApplication")
@Api(description = "投保单")
public class InsuranceApplicationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private JobsService jobsService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceApplication> create(){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        InsuranceApplication insuranceApplication = new InsuranceApplication();

        msgResult.setData(insuranceApplication);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceApplication> add(@RequestBody InsuranceApplication insuranceApplication,@RequestAttribute String subject){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            insuranceApplication.setId(UUID.randomUUID().toString());
            insuranceApplication.setDelFlag(false);
            insuranceApplication.setCreateBy(subject);
            insuranceApplication.setCreateTime(new Date());
            
            int affectCount = insuranceApplicationService.insert(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceApplication> edit(@PathVariable("id") String id){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            if (insuranceApplication != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="获取投保信息")
    @PostMapping("insureDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "投保信息ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="no",value = "投保份数（比如:第几份）",required = true,paramType = "query"),
    })
    public MessageResult<InsuranceApplication> insureDetail(String id,String no){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            List<InsurancePolicy> list = new ArrayList<InsurancePolicy>();

            int index = 0;

            if(StringUtils.isNotEmpty(no)){
                index = Integer.parseInt(no);
            }

            if(index==1){
                list = applicationPolicyService.findFirstPolicyByApplicationId(id);
            }
            else{
                list = applicationPolicyService.findNewPolicyByApplicationId(id);
            }


            if(list.size()>0){
                insuranceApplication.setInsurancePolicy(list.get(0));
            }

            if (insuranceApplication != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceApplication> update(@RequestBody InsuranceApplication insuranceApplication,@RequestAttribute String subject){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            insuranceApplication.setUpdateBy(subject);
            insuranceApplication.setUpdateTime(new Date());
            
            int affectCount = insuranceApplicationService.update(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);
            insuranceApplication.setDelFlag(true);
            insuranceApplication.setUpdateBy(subject);
            insuranceApplication.setUpdateTime(new Date());

            int affectCount = insuranceApplicationService.update(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);
                insuranceApplication.setDelFlag(true);
                insuranceApplication.setUpdateBy(subject);
                insuranceApplication.setUpdateTime(new Date());

                affectCount += insuranceApplicationService.update(insuranceApplication);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="status",value = "状态",paramType = "query"),
            @ApiImplicitParam(name="definitionId",value = "险种ID",paramType = "query"),
            @ApiImplicitParam(name="insuredName",value = "投保人姓名",paramType = "query"),
            @ApiImplicitParam(name="startTime",value = "开始时间",paramType = "query"),
            @ApiImplicitParam(name="endTime",value = "结束时间",paramType = "query"),
            @ApiImplicitParam(name="userType",value = "用户类型（0为人才超市，1为PICC）",paramType = "query")
    })
    public MessageResult<Map> pageList(
            String status,String definitionId,String insuredName,String startTime,String endTime,String userType,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        if (StringUtils.isNotEmpty(definitionId)) {
            searchParams.put("definitionId",definitionId);
        }

        if (StringUtils.isNotEmpty(insuredName)) {
            searchParams.put("insuredName","%"+insuredName+"%");
        }

        if (StringUtils.isNotEmpty(startTime)) {
            searchParams.put("startTime",startTime);
        }

        if (StringUtils.isNotEmpty(endTime)) {
            searchParams.put("endTime",endTime);
        }

        if (StringUtils.isNotEmpty(userType)) {
            if(userType.equals("1")){
                searchParams.put("userType",userType);
            }
        }

        Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="definitionId",value = "险种ID",required = true,paramType = "query")
    })
    public MessageResult<Map> acceptDetail(String definitionId, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> map = new HashMap<String,Object>();

        InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(definitionId);

        map.put("id",insuranceDefinition.getId());
        map.put("name",insuranceDefinition.getName());

        List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(definitionId);

        List<Map<String,Object>> mapList = new ArrayList<Map<String,Object>>();

        for(InsuranceDefinitionLimit  insuranceDefinitionLimit:insuranceDefinitionLimitList){
            Map<String,Object> map1 = new HashMap<String,Object>();
            map1.put("id",insuranceDefinitionLimit.getId());
            map1.put("name",insuranceDefinitionLimit.getName());
            map1.put("limit",insuranceDefinitionLimit.getLimit());
            map1.put("unit",insuranceDefinitionLimit.getUnit());
            mapList.add(map1);
        }

        map.put("insuranceDefinitionLimitList",mapList);

        List<Jobs> jobsList = jobsService.list();

        List<Map<String,Object>> mapList1 = new ArrayList<Map<String,Object>>();

        for (Jobs jobs:jobsList) {
            Map<String,Object> map1 = new HashMap<String,Object>();
            String jobId = jobs.getId();
            map1.put("id",jobId);
            map1.put("name",jobs.getName());
            map1.put("content",jobs.getContent());

            List<InsuranceJobs> insuranceJobsList = insuranceJobsService.findByDefinitionIdAndJobId(definitionId,jobId);

            List<Map<String,Object>> mapList2 = new ArrayList<Map<String,Object>>();

            for (InsuranceJobs insuranceJobs:insuranceJobsList) {
                Map<String,Object> map2 = new HashMap<String,Object>();
                map2.put("id",insuranceJobs.getId());
                map2.put("charges",insuranceJobs.getCharges());

                mapList2.add(map2);
            }

            map1.put("insuranceJobsList",mapList2);

            mapList1.add(map1);
        }

        map.put("jobsList",mapList1);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }
}
