package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceAgent;
import com.jpsoft.picc.modules.base.service.InsuranceAgentService;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.joda.time.DateTime;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/insurancePolicy")
@Api(description = "每月投保单")
public class InsurancePolicyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsuranceAgentService insuranceAgentService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsurancePolicy> create(){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = new InsurancePolicy();

        msgResult.setData(insurancePolicy);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsurancePolicy> add(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setId(UUID.randomUUID().toString());
            insurancePolicy.setDelFlag(false);
            insurancePolicy.setCreateBy(subject);
            insurancePolicy.setCreateTime(new Date());
            
            int affectCount = insurancePolicyService.insert(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsurancePolicy> edit(@PathVariable("id") String id){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

            if (insurancePolicy != null) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsurancePolicy> update(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());
            
            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
            insurancePolicy.setDelFlag(true);
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
                insurancePolicy.setDelFlag(true);
                insurancePolicy.setUpdateBy(subject);
                insurancePolicy.setUpdateTime(new Date());

                affectCount += insurancePolicyService.update(insurancePolicy);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsurancePolicy> page = insurancePolicyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="读取附件")
    @RequestMapping(value = "attachmentList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> attachmentList(String id, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,String> map = new HashMap<String,String>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

        String file = insurancePolicy.getInsurancePolicyFile();

        String[] files = file.split(";");

        if(files.length==2){
            map.put("file1",files[0]);
            map.put("file2",files[1]);
        }

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }



    @ApiOperation(value="读取推广信息")
    @RequestMapping(value = "loadInsuranceAgent",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<InsuranceAgent> loadInsuranceAgent(String id, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

        String agentId = insurancePolicy.getAgentId();

        InsuranceAgent insuranceAgent = insuranceAgentService.get(agentId);

        msgResult.setResult(true);
        msgResult.setData(insuranceAgent);

        return msgResult;
    }


    @ApiOperation(value="人才超市初审")
    @RequestMapping(value = "firstApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> firstApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status = String.valueOf(PolicyStatus.PendingRetrial.getValue());

        int affectCount = approval(applicationId,policyId,processStatus,opinion,status ,subject);

        msgResult.setResult(true);
        msgResult.setData(affectCount);

        return msgResult;
    }


    @ApiOperation(value="PICC复审")
    @RequestMapping(value = "secondApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> secondApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status = String.valueOf(PolicyStatus.PendingPay.getValue());

        int affectCount = approval(applicationId,policyId,processStatus,opinion,status ,subject);

        msgResult.setResult(true);
        msgResult.setData(affectCount);

        return msgResult;
    }


    @ApiOperation(value="PICC出单")
    @RequestMapping(value = "issued",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> issued(String applicationId,String policyId,String processStatus,String opinion,@RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status = String.valueOf(PolicyStatus.SendOutPolicy.getValue());

        int affectCount = approval(applicationId,policyId,processStatus,opinion,status ,subject);

        boolean flag = false;
        String msg = "";

        if(affectCount>0){
            //生成下月投保单

            try {
                generatePolicyInTrans(applicationId, policyId, subject);
                flag = true;
            }
            catch (Exception ex){
                flag = false;
                msg = ex.getMessage();
            }
        }

        msgResult.setResult(flag);
        msgResult.setData(affectCount);
        msgResult.setMessage(msg);

        return msgResult;
    }



    @ApiOperation(value="企业缴费")
    @RequestMapping(value = "pay",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> pay(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status = String.valueOf(PolicyStatus.PendingMakePolicy.getValue());

        int affectCount = approval(applicationId,policyId,processStatus,opinion,status ,subject);

        msgResult.setResult(true);
        msgResult.setData(affectCount);

        return msgResult;
    }



    @ApiOperation(value="人才超市与PICC转账")
    @RequestMapping(value = "transfer",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="accountBank",value = "入账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="issuingBank",value = "出账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="singlNumber",value = "单号",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> transfer(String applicationId,String policyId,
                                           String accountBank,String issuingBank,String singlNumber,
                                           String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status = String.valueOf(PolicyStatus.PendingOutPolicy.getValue());

        InsurancePolicy item = insurancePolicyService.get(policyId);

        item.setAccountBank(accountBank);
        item.setIssuingBank(issuingBank);
        item.setSinglNumber(singlNumber);
        item.setTransferTime(new Date());

        int affectCount = insurancePolicyService.update(item);

        int affectCount1 = 0;

        if(affectCount>0){
            affectCount1 = approval(applicationId,policyId,processStatus,opinion,status ,subject);
        }

        msgResult.setResult(true);
        msgResult.setData(affectCount1);

        return msgResult;
    }


    private Integer approval(String applicationId,String id,String processStatus,String opinion,String status,String subject){

        int affectCount = 0;

        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

        List<InsurancePolicy> insurancePolicyList = applicationPolicyService.findNewPolicyByApplicationId(applicationId);

        if(insurancePolicyList.size()>0){
            int no = insurancePolicyList.get(0).getNo();

            //如果每月投保单为第一份，就修改投保单状态
            if(no == 1){
                insuranceApplication.setStatus(status);
                insuranceApplicationService.update(insuranceApplication);
            }

            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

            insurancePolicy.setProcessStatus(processStatus);
            insurancePolicy.setStatus(status);
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount1 = insurancePolicyService.update(insurancePolicy);

            if(affectCount1 > 0){
                InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();
                insurancePolicyRecord.setPolicyId(insurancePolicy.getId());
                insurancePolicyRecord.setStatus(status);
                insurancePolicyRecord.setOpinion(opinion);
                insurancePolicyRecord.setProcessStatus(processStatus);
                insurancePolicyRecord.setId(UUID.randomUUID().toString());
                insurancePolicyRecord.setDelFlag(false);
                insurancePolicyRecord.setCreateBy(subject);
                insurancePolicyRecord.setCreateTime(new Date());

                affectCount = insurancePolicyRecordService.insert(insurancePolicyRecord);
            }
        }

        return affectCount;
    }





    @Transactional(rollbackFor = Exception.class)
    private void generatePolicyInTrans(String applicationId, String policyId, String subject) throws Exception {
        //添加下月投保单
        InsurancePolicy srcPolicy = insurancePolicyService.get(policyId);
        srcPolicy.setStatus(PolicyStatus.SendOutPolicy.getValue() + "");
        srcPolicy.setUpdateTime(new Date());

        insurancePolicyService.update(srcPolicy);

        InsurancePolicy destPolicy = new InsurancePolicy();

        PojoUtils.map(srcPolicy,destPolicy);

        DateTime startTime = new DateTime(srcPolicy.getEndTime()).plusDays(1);

        destPolicy.setStartTime(startTime.toDate());
        destPolicy.setEndTime(startTime.plusMonths(1).toDate());

        destPolicy.setEffectiveDate(startTime.toString("yyyyMM"));

        boolean exist = insurancePolicyService.existByApplicationIdAndEffectiveDate(applicationId,destPolicy.getEffectiveDate());

        if (exist){
            throw new Exception(destPolicy.getEffectiveDate() + "投保单已存在!");
        }

        destPolicy.setNo(srcPolicy.getNo()+1);
        destPolicy.setId(UUID.randomUUID().toString());
        destPolicy.setStatus(PolicyStatus.Draft.getValue() + "");
        destPolicy.setCreateBy(subject);
        destPolicy.setCreateTime(new Date());
        destPolicy.setUpdateBy(subject);
        destPolicy.setUpdateTime(new Date());

        insurancePolicyService.insert(destPolicy);

        //关联投保单及每月投保单
        ApplicationPolicy applicationPolicy = new ApplicationPolicy();
        applicationPolicy.setId(UUID.randomUUID().toString());
        applicationPolicy.setApplicationId(applicationId);
        applicationPolicy.setPolicyId(destPolicy.getId());

        applicationPolicyService.insert(applicationPolicy);

        //关联参保人员
        List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(policyId);

        for (InsurancePolicyMember srcMember : memberList) {
            InsurancePolicyMember destMember = new InsurancePolicyMember();

            destMember.setId(UUID.randomUUID().toString());
            destMember.setPolicyId(destPolicy.getId());
            destMember.setMemberId(srcMember.getMemberId());
            destMember.setDelFlag(false);
            destMember.setCreateBy(subject);
            destMember.setCreateTime(new Date());
            destMember.setStatus(srcMember.getStatus());

            insurancePolicyMemberService.insert(destMember);
        }
    }
}
