package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.Message;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.MessageService;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Api(description="消息列表")
@RestController
@RequestMapping("/auth/message")
public class MessageController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MessageService messageService;

    @Autowired
    private CompanyService companyService;

    private Boolean read(String id,String updateBy){
        Boolean bl = false;
        try{
            Message message = messageService.get(id);
            message.setStatus(true);
            message.setUpdateBy(updateBy);
            message.setUpdateTime(new Date());

            int affectCount = messageService.update(message);

            if (affectCount > 0) {
                bl = true;
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
        }

        return bl;
    }

    @ApiOperation(value="消息分页列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status",value = "状态(未读false/已读true)", required = false, paramType = "form",dataType = "Boolean")
    })
    public MessageResult<Map> pageList(@RequestParam(value="status",defaultValue="") Boolean status,
                                       @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
                                       @RequestParam(value="pageSize",defaultValue="20") int pageSize,
                                       HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("recipientId",company.getId());

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (status != null) {
            searchParams.put("status",status);
        }

        Page<Message> page = messageService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="未读消息数量")
    @GetMapping(value = "unReadCount")
    public MessageResult<Long> unReadCount(HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());
        MessageResult<Long> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("recipientId",company.getId());

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));
        searchParams.put("status",false);

        Page<Message> page = messageService.pageSearch(searchParams,1,1,sortList);

        msgResult.setResult(true);
        msgResult.setData(page.getTotal());

        return msgResult;
    }

    @ApiOperation(value="全部消息设为已读")
    @PostMapping("allRead")
    public MessageResult<String> allRead(HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("recipientId",company.getId());
            searchParams.put("status",false);
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));
            Page<Message> page = messageService.pageSearch(searchParams,1,1000,sortList);
            for(Message message : page.getResult()){
                if(read(message.getId(),principal.getName())){
                    affectCount++;
                }
            }

            msgResult.setResult(true);
            msgResult.setData("共计"+affectCount+"条消息设为已读。");
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取消息信息")
    @RequestMapping(value = "detail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Message> detail(@RequestParam(value="id",defaultValue="") String id,
                                         HttpServletRequest request){
        MessageResult<Message> messageResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            Company company = companyService.findByCreateBy(principal.getName());

            Message message = messageService.get(id);

            if(read(id,principal.getName())) {
                messageResult.setResult(true);
                messageResult.setData(message);
            }
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
