package com.jpsoft.picc.modules.pub.controller;

import cn.hutool.json.JSONObject;
import com.jpsoft.picc.config.JpCloudConfig;
import com.jpsoft.picc.modules.base.entity.TransactionRecord;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.TransactionRecordService;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.SignUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2020-2-11 12:37
 */

@Controller
@RequestMapping("/pay")
@Api(description="支付接口")
@Slf4j
public class PayController {
    @Autowired
    private JpCloudConfig jpCloudConfig;

    @Autowired
    private TransactionRecordService transactionRecordService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private CompanyService companyService;

    @PostMapping(value = "/orderNotifyBack")
    @ResponseBody
    public Map orderNotifyBack(HttpServletRequest req,String payTime,String payType,String orderNo,String sign) {
        HashMap<String, Object> retMap = new HashMap<String, Object>();

        String status = "success";
        String message = "订单支付成功";
        JSONObject body = new JSONObject();
        log.warn("payTime>>>" + payTime + "paymentType>>>" + payType + ">>>>orderNo>>" + orderNo + "sign>>>" + sign);
        if(StringUtils.isNotBlank(payType) && StringUtils.isNotBlank(orderNo)){

            HashMap<String,Object> signMap = new HashMap<String,Object>();

            Map<String,String[]> paramMap = req.getParameterMap();

            for (Map.Entry<String, String[]> map : paramMap.entrySet()) {
                if(!"sign".equals(map.getKey())){
                    signMap.put(map.getKey(), map.getValue()[0]);
                }
            }
      //      String signKey = ReadPropertiesUtils.getProperty("thirdInfo", "signKey");
            boolean result = SignUtil.validateSign(signMap,"1234567890",sign);

            if(result){
                TransactionRecord transactionRecord = transactionRecordService.getByOrderNo(orderNo);

                InsuranceApplication insuranceApplication = insuranceApplicationService.get(transactionRecord.getApplicationId());

                if(transactionRecord != null && !"20".equals(transactionRecord.getPaymentStatus())){
                    transactionRecord.setPaymentStatus("20");
                    transactionRecord.setPaymentTime(new Date());
                    transactionRecord.setBuyType(payType);
                    transactionRecord.setUpdateTime(new Date());

                    transactionRecordService.update(transactionRecord);

                    //将投保单状态改为待制单
                    if(insuranceApplication!=null) {
                        insuranceApplication.setStatus(PolicyStatus.PendingMakePolicy.getValue() + "");
                        insuranceApplication.setUpdateTime(new Date());

                        insuranceApplicationService.update(insuranceApplication);
                    }

                    body.put("orderNo", orderNo);
                    body.put("backStatus", "回调成功");
                    body.put("backTime", new Date().getTime()/1000);
                }else if(transactionRecord == null){
                    message = "订单不存在";
                    status = "fail";
                }else if("20".equals(transactionRecord.getPaymentStatus())){
                    message = "订单已支付";
                    status = "fail";
                }
            }else{
                message = "签名不正确";
                status = "fail";
            }

        }else{
            message = "订单号不存在";
            status = "fail";
        }

        retMap.put("status", status);
        retMap.put("message", message);
        retMap.put("body", body);

        return retMap;
    }

    @GetMapping(value="/paySuccess")
    @ApiOperation(value="返回前端")
    public String paySuccess(){
        return "redirect:/portal/";
    }



}
